@@ -41,6 +41,9 @@
 /* ZLib buffer settings */
 #define ZLIB_BUF_SIZE		      8192
 
+/* Required minimum space needed to create databases (5Mb) */
+#define TRACKER_DB_MIN_REQUIRED_SPACE 5242880
+
 /* Default memory settings for databases */
 #define TRACKER_DB_PAGE_SIZE_DONT_SET -1
 
@@ -48,7 +51,7 @@
 #define TRACKER_DB_MAX_FILE_SIZE      2000000000 
 
 /* Set current database version we are working with */
-#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_3
+#define TRACKER_DB_VERSION_NOW        TRACKER_DB_VERSION_4
 #define TRACKER_DB_VERSION_FILE       "db-version.txt"
 
 typedef enum {
@@ -61,8 +64,9 @@ typedef enum {
 	TRACKER_DB_VERSION_UNKNOWN, /* Unknown */
 	TRACKER_DB_VERSION_1,       /* Version 0.6.6  (before indexer-split) */
 	TRACKER_DB_VERSION_2,       /* Version 0.6.90 (after  indexer-split) */
-	TRACKER_DB_VERSION_3,       /* Version 0.6.91 (current TRUNK) */
-	TRACKER_DB_VERSION_4        /* Version 0.7    (vstore branch) */
+	TRACKER_DB_VERSION_3,       /* Version 0.6.91 (stable release) */
+	TRACKER_DB_VERSION_4,       /* Version 0.6.92 (current TRUNK) */
+	TRACKER_DB_VERSION_5        /* Version 0.7    (vstore branch) */
 } TrackerDBVersion;
 
 typedef struct {
@@ -133,6 +137,7 @@ static TrackerDBDefinition dbs[] = {
 	  FALSE,
 	  FALSE,
  	  0 },
+#ifdef HAVE_SQLITE_FTS
 	{ TRACKER_DB_FILE_FULLTEXT,
 	  TRACKER_DB_LOCATION_DATA_DIR,
 	  NULL,
@@ -145,6 +150,7 @@ static TrackerDBDefinition dbs[] = {
 	  FALSE,
 	  TRUE,
  	  0 },
+#endif /* HAVE_SQLITE_FTS */
 	{ TRACKER_DB_FILE_CONTENTS,
 	  TRACKER_DB_LOCATION_DATA_DIR,
 	  NULL,
@@ -168,6 +174,7 @@ static TrackerDBDefinition dbs[] = {
 	  TRUE,
 	  FALSE,
  	  0 },
+#ifdef HAVE_SQLITE_FTS
 	{ TRACKER_DB_EMAIL_FULLTEXT,
 	  TRACKER_DB_LOCATION_DATA_DIR,
 	  NULL,
@@ -180,6 +187,7 @@ static TrackerDBDefinition dbs[] = {
 	  FALSE,
 	  TRUE,
  	  0 },
+#endif /* HAVE_SQLITE_FTS */
 	{ TRACKER_DB_EMAIL_CONTENTS,
 	  TRACKER_DB_LOCATION_DATA_DIR,
 	  NULL,
@@ -194,21 +202,25 @@ static TrackerDBDefinition dbs[] = {
  	  0 },
 };
 
-static gboolean		   db_exec_no_reply    (TrackerDBInterface *iface,
-						const gchar	   *query,
-						...);
-static TrackerDBInterface *db_interface_create (TrackerDB	    db);
+static gboolean            db_exec_no_reply          (TrackerDBInterface *iface,
+						      const gchar        *query,
+						      ...);
+static TrackerDBInterface *db_interface_create       (TrackerDB           db);
+static gboolean            save_pragma_file_defaults (gboolean            safe);
 
 static gboolean		   initialized;
 static GHashTable	  *prepared_queries;
+static GHashTable         *pragmas;
 static gchar		  *services_dir;
 static gchar		  *sql_dir;
+static gchar		  *config_dir;
 static gchar		  *data_dir;
 static gchar		  *user_data_dir;
 static gchar		  *sys_tmp_dir;
 static gpointer		   db_type_enum_class_pointer;
 static TrackerDBInterface *file_iface;
 static TrackerDBInterface *email_iface;
+static GList              *not_owned_ifaces = NULL;
 
 static const gchar *
 location_to_directory (TrackerDBLocation location)
@@ -225,6 +237,149 @@ location_to_directory (TrackerDBLocation location)
 	};
 }
 
+static gboolean
+load_pragma_file (const gchar *profile_name)
+{
+	GKeyFile      *key_file = NULL;
+	GError        *error = NULL;
+	GStrv          keys;
+	gchar	      *pragma_file;
+	gint           i;
+
+	if (pragmas) {
+		g_hash_table_unref (pragmas);
+	}
+
+	pragmas = g_hash_table_new_full (g_str_hash,
+					 g_str_equal,
+					 g_free,
+					 g_free);
+
+	key_file = g_key_file_new ();
+	pragma_file = g_build_filename (config_dir, "sqlite-db.pragmas", NULL);
+
+	if (!profile_name) {
+		profile_name = "Safe";
+	}
+
+	g_message ("Loading pragma file:'%s' using profile '%s'",
+		   pragma_file,
+		   profile_name);
+
+	if (!g_key_file_load_from_file (key_file, pragma_file, G_KEY_FILE_NONE, &error)) {
+		g_message ("  Couldn't load pragma file, %s", 
+			    error ? error->message : "no error given");
+
+		g_clear_error (&error);
+		g_free (pragma_file);
+		g_key_file_free (key_file);
+
+		g_message ("  Trying to re-create file with defaults"); 
+
+		save_pragma_file_defaults (TRUE);
+		return load_pragma_file (NULL);
+	}
+
+	if (!g_key_file_has_group (key_file, profile_name)) {
+		g_warning ("  Profile '%s' does not exist, stepping back to 'Safe'", profile_name);
+		profile_name = "Safe";
+	}
+
+	keys = g_key_file_get_keys (key_file, profile_name, NULL, NULL);
+
+	for (i = 0; keys[i]; i++) {
+		gchar *value;
+
+		value = g_key_file_get_string (key_file, profile_name, keys[i], NULL);
+		g_hash_table_insert (pragmas, g_strdup (keys[i]), value);
+
+		g_message ("  Adding pragma '%s' with value '%s'", 
+			   keys[i],
+			   value);
+	}
+
+	g_strfreev (keys);
+
+	g_free (pragma_file);
+	g_key_file_free (key_file);
+
+	return TRUE;
+}
+
+static gboolean
+save_pragma_file_defaults (gboolean safe)
+{
+	GKeyFile      *key_file = NULL;
+	GError        *error = NULL;
+	gchar	      *pragma_file;
+	gchar         *content;
+	const gchar   *group;
+
+	pragma_file = g_build_filename (config_dir, "sqlite-db.pragmas", NULL);
+	key_file = g_key_file_new ();
+
+	g_message ("Saving pragma file:'%s' with defaults using %s values", 
+		   pragma_file,
+		   safe ? "safe" : "fast");
+
+	g_key_file_set_comment (key_file, NULL, NULL,
+				"\n"
+				" There are two groups here, \"Safe\" and \"Fast\".\n"
+				" The \"Safe\" group is the default.\n"
+				"\n"
+				" These are the values which each property can be set to:\n"
+				"\n"
+				"   encoding:      \"UTF-8\", \"UTF-16\", \"UTF-16le\" or \"UTF-16be\"\n"
+				"   journal_mode:  \"OFF\", \"TRUNCATE\", \"PERSIST\", \"MEMORY\" or \"DELETE\"\n"
+				"   synchronous:   \"OFF\", \"NORMAL\", or \"FULL\"\n"
+				"   temp_store:    \"DEFAULT\", \"FILE\", or \"MEMORY\"\n"
+				"   auto_vacuum:   \"NONE\", \"FULL\", or \"INCREMENTAL\"\n"
+				"   count_changes: \"0\" or \"1\"\n",
+				NULL);
+
+	group = "Safe";
+	g_key_file_set_string (key_file, group, "encoding", "\"UTF-8\"");
+	g_key_file_set_string (key_file, group, "journal_mode", "DELETE");
+	g_key_file_set_string (key_file, group, "synchronous", "NORMAL");
+	g_key_file_set_string (key_file, group, "temp_store", "MEMORY");
+	g_key_file_set_string (key_file, group, "auto_vacuum", "NONE");
+	g_key_file_set_string (key_file, group, "count_changes", "0");
+
+	group = "Fast";
+	g_key_file_set_string (key_file, group, "encoding", "\"UTF-8\"");
+	g_key_file_set_string (key_file, group, "journal_mode", "MEMORY");
+	g_key_file_set_string (key_file, group, "synchronous", "OFF");
+	g_key_file_set_string (key_file, group, "temp_store", "MEMORY");
+	g_key_file_set_string (key_file, group, "auto_vacuum", "NONE");
+	g_key_file_set_string (key_file, group, "count_changes", "0");
+
+	content = g_key_file_to_data (key_file, NULL, &error);
+	g_key_file_free (key_file);
+
+	if (error) {
+		g_critical ("Couldn't produce default pragma file, %s", 
+			    error->message);
+		g_clear_error (&error);
+		g_free (pragma_file);
+		return FALSE;
+	}
+
+	if (!g_file_set_contents (pragma_file, content, -1, &error)) {
+		g_critical ("Couldn't write default configuration, %s", 
+			    error->message);
+		g_clear_error (&error);
+		g_free (content);
+		g_free (pragma_file);
+		return FALSE;
+	}
+
+	g_message ("  Written");
+	g_free (content);
+	g_free (pragma_file);
+	
+	return TRUE;
+}
+
 static void
 load_sql_file (TrackerDBInterface *iface,
 	       const gchar	  *file,
@@ -282,15 +437,21 @@ load_metadata_file (TrackerDBInterface *iface,
 		    const gchar        *filename)
 {
 	GKeyFile      *key_file = NULL;
+	GError        *error = NULL;
 	gchar	      *service_file, *str_id;
 	gchar	     **groups, **keys;
 	TrackerField  *def;
 	gint	       id, i, j;
 
+	g_message ("Loading metadata file '%s'", filename);
+
 	key_file = g_key_file_new ();
 	service_file = g_build_filename (services_dir, filename, NULL);
 
-	if (!g_key_file_load_from_file (key_file, service_file, G_KEY_FILE_NONE, NULL)) {
+	if (!g_key_file_load_from_file (key_file, service_file, G_KEY_FILE_NONE, &error)) {
+		g_critical ("Couldn't load service file, %s", 
+			    error ? error->message : "no error given");
+		g_clear_error (&error);
 		g_free (service_file);
 		g_key_file_free (key_file);
 		return;
@@ -302,6 +463,7 @@ load_metadata_file (TrackerDBInterface *iface,
 		def = tracker_ontology_get_field_by_name (groups[i]);
 
 		if (!def) {
+			g_message ("  Adding ontology metadata:'%s'", groups[i]);
 			tracker_db_interface_execute_procedure (iface,
 								NULL,
 								"InsertMetadataType",
@@ -348,15 +510,17 @@ load_metadata_file (TrackerDBInterface *iface,
 					g_critical ("Field '%s' doesn't have a valid data type '%s'", groups[i], new_value);
 				}
 			} else {
-				gchar *esc_value;
+				gchar *escaped_value;
 
-				esc_value = tracker_escape_string (new_value);
+				escaped_value = tracker_escape_db_string (new_value, TRUE, FALSE);
 
 				tracker_db_interface_execute_query (iface, NULL,
-								    "update MetaDataTypes set  %s = '%s' where ID = %d",
-								    keys[j], esc_value, id);
+								    "update MetaDataTypes set  %s = %s where ID = %d",
+								    keys[j], 
+								    escaped_value, 
+								    id);
 
-				g_free (esc_value);
+				g_free (escaped_value);
 			}
 
 			g_free (new_value);
@@ -377,15 +541,21 @@ load_service_file (TrackerDBInterface *iface,
 {
 	TrackerService	*service;
 	GKeyFile	*key_file = NULL;
+	GError          *error = NULL;
 	gchar		*service_file, *str_id;
 	gchar	       **groups, **keys;
 	gint		 i, j, id;
 
+	g_message ("Loading service file '%s'", filename);
+
 	service_file = g_build_filename (services_dir, filename, NULL);
 
 	key_file = g_key_file_new ();
 
-	if (!g_key_file_load_from_file (key_file, service_file, G_KEY_FILE_NONE, NULL)) {
+	if (!g_key_file_load_from_file (key_file, service_file, G_KEY_FILE_NONE, &error)) {
+		g_critical ("Couldn't load service file, %s", 
+			    error ? error->message : "no error given");
+		g_clear_error (&error);
 		g_free (service_file);
 		g_key_file_free (key_file);
 		return;
@@ -394,10 +564,10 @@ load_service_file (TrackerDBInterface *iface,
 	groups = g_key_file_get_groups (key_file, NULL);
 
 	for (i = 0; groups[i]; i++) {
-		g_message ("Trying to obtain service:'%s' in cache", groups[i]);
 		service = tracker_ontology_get_service_by_name (groups[i]);
 
 		if (!service) {
+			g_message ("Adding ontology service type:'%s'", groups[i]);
 			tracker_db_interface_execute_procedure (iface,
 								NULL,
 								"InsertServiceType",
@@ -501,20 +671,52 @@ load_service_file (TrackerDBInterface *iface,
 
 				g_strfreev (tab_array);
 			} else {
-				gchar *value, *new_value, *esc_value;
+				gchar *value, *new_value, *escaped_value;
 
 				value = g_key_file_get_string (key_file, groups[i], keys[j], NULL);
 				new_value = tracker_string_boolean_to_string_gint (value);
-				esc_value = tracker_escape_string (new_value);
+				escaped_value = tracker_escape_db_string (new_value, TRUE, FALSE);
 
+				/* Special case "Parent */
+				if (g_ascii_strcasecmp (keys[j], "parent") == 0) {
+					TrackerDBResultSet *result_set;
+					gchar *query;
+
+					query = g_strdup_printf ("SELECT TypeId FROM ServiceTypes WHERE TypeName = %s",
+								 escaped_value);
+					result_set = tracker_db_interface_execute_query (iface, NULL, "%s", query);
+					g_free (query);
+
+					if (result_set) {
+						GValue value = {0, };
+						GValue transform = {0, };
+
+						g_value_init (&transform, G_TYPE_STRING);
+						
+						_tracker_db_result_set_get_value (result_set, 0, &value);
+						if (g_value_transform (&value, &transform)) {
+							tracker_db_interface_execute_query (iface,
+											    NULL,
+											    "UPDATE ServiceTypes SET ParentId = '%s' WHERE TypeID = %s",
+											    g_value_get_string (&transform),
+											    str_id);
+
+						}
+						
+						g_value_unset (&value);
+						g_value_unset (&transform);
+						g_object_unref (result_set);
+					}
+				}
+				
 				tracker_db_interface_execute_query (iface,
 								    NULL,
-								    "update ServiceTypes set  %s = '%s' where TypeID = %s",
+								    "UPDATE ServiceTypes SET %s = %s WHERE TypeID = %s",
 								    keys[j],
-								    esc_value,
+								    escaped_value,
 								    str_id);
 
-				g_free (esc_value);
+				g_free (escaped_value);
 				g_free (value);
 				g_free (new_value);
 			}
@@ -529,24 +731,6 @@ load_service_file (TrackerDBInterface *iface,
 	g_free (service_file);
 }
 
-static TrackerDBResultSet *
-db_exec_proc (TrackerDBInterface *iface,
-	      const gchar	 *procedure,
-	      ...)
-{
-	TrackerDBResultSet *result_set;
-	va_list		    args;
-
-	va_start (args, procedure);
-	result_set = tracker_db_interface_execute_vprocedure (iface,
-							      NULL,
-							      procedure,
-							      args);
-	va_end (args);
-
-	return result_set;
-}
-
 static gboolean
 db_exec_no_reply (TrackerDBInterface *iface,
 		  const gchar	     *query,
@@ -581,6 +765,15 @@ load_prepared_queries (void)
 	gchar	    *filename;
 	gdouble      secs;
 
+	if (prepared_queries) {
+		g_hash_table_unref (prepared_queries);
+	}
+
+	prepared_queries = g_hash_table_new_full (g_str_hash,
+						  g_str_equal,
+						  g_free,
+						  g_free);
+
 	g_message ("Loading prepared queries...");
 
 	filename = g_build_filename (sql_dir, "sqlite-stored-procs.sql", NULL);
@@ -690,11 +883,11 @@ load_prepared_queries (void)
 				continue;
 			}
 
-			g_message ("  Adding query:'%s'", details[0]);
+			g_message ("  Adding query:'%s'", g_strstrip (details[0]));
 
 			g_hash_table_insert (prepared_queries,
-					     g_strdup (details[0]),
-					     g_strdup (details[1]));
+					     g_strdup (g_strstrip (details[0])),
+					     g_strdup (g_strstrip (details[1])));
 			g_strfreev (details);
 		}
 
@@ -1278,7 +1471,7 @@ function_collate_key (TrackerDBInterface *interface,
 	GValue result = { 0 };
 	gchar *collate_key;
 
-	collate_key = g_utf8_collate_key (g_value_get_string (&values[0]), -1);
+	collate_key = g_utf8_collate_key_for_filename (g_value_get_string (&values[0]), -1);
 
 	g_value_init (&result, G_TYPE_STRING);
 	g_value_take_string (&result, collate_key);
@@ -1292,11 +1485,25 @@ db_set_params (TrackerDBInterface *iface,
 	       gint		   page_size,
 	       gboolean		   add_functions)
 {
-	tracker_db_interface_execute_query (iface, NULL, "PRAGMA synchronous = NORMAL;");
-	tracker_db_interface_execute_query (iface, NULL, "PRAGMA count_changes = 0;");
-	tracker_db_interface_execute_query (iface, NULL, "PRAGMA temp_store = FILE;");
-	tracker_db_interface_execute_query (iface, NULL, "PRAGMA encoding = \"UTF-8\"");
-	tracker_db_interface_execute_query (iface, NULL, "PRAGMA auto_vacuum = 0;");
+	GHashTableIter  iter;
+	gpointer        key, value;
+
+	if (pragmas) {
+		g_hash_table_iter_init (&iter, pragmas);
+		while (g_hash_table_iter_next (&iter, &key, &value)) {
+			TrackerDBResultSet *result_set;
+
+			result_set = tracker_db_interface_execute_query (iface,
+									 NULL,
+									 "PRAGMA %s = %s;",
+									 (const gchar*) key,
+									 (const gchar*) value);
+
+			if (result_set) {
+				g_object_unref (result_set);
+			}
+		}
+	}
 
 	if (page_size != TRACKER_DB_PAGE_SIZE_DONT_SET) {
 		g_message ("  Setting page size to %d", page_size);
@@ -1404,10 +1611,6 @@ db_get_static_data (TrackerDBInterface *iface)
 				g_slist_free (child_ids);
 			}
 
-			g_message ("Loading metadata def:'%s' with weight:%d",
-				   tracker_field_get_name (def),
-				   tracker_field_get_weight (def));
-
 			tracker_ontology_field_add (def);
 			g_object_unref (def);
 
@@ -1444,14 +1647,14 @@ db_get_static_data (TrackerDBInterface *iface)
 			mimes = db_get_mimes_for_service_id (iface, id);
 			mime_prefixes = db_get_mime_prefixes_for_service_id (iface, id);
 
-			g_message ("Adding service:'%s' with id:%d and mimes:%d",
+			g_message ("Loading ontology service:'%s' with id:%d and mimes:%d",
 				   name,
 				   id,
 				   g_slist_length (mimes));
 
 			tracker_ontology_service_add (service,
-							   mimes,
-							   mime_prefixes);
+						      mimes,
+						      mime_prefixes);
 
 			g_slist_free (mimes);
 			g_slist_free (mime_prefixes);
@@ -1464,7 +1667,6 @@ db_get_static_data (TrackerDBInterface *iface)
 	}
 }
 
-
 static const gchar *
 db_type_to_string (TrackerDB db)
 {
@@ -1529,6 +1731,7 @@ db_interface_get_common (void)
 		const gchar *conf_file;
 
 		tracker_db_interface_start_transaction (iface);
+
 		/* Create tables */
 		load_sql_file (iface, "sqlite-tracker.sql", NULL);
 		load_sql_file (iface, "sqlite-metadata.sql", NULL);
@@ -1553,12 +1756,10 @@ db_interface_get_common (void)
 			}
 
 			if (g_str_has_suffix (conf_file, ".service")) {
-				g_debug ("Loading service file %s", conf_file);
 				load_service_file (iface, conf_file);
 			}
 
 			if (g_str_has_suffix (conf_file, ".metadata")) {
-				g_debug ("Loading metadata file %s", conf_file);
 				load_metadata_file (iface, conf_file);
 			}
 
@@ -1611,6 +1812,8 @@ db_interface_get_file_metadata (void)
 	return iface;
 }
 
+#ifdef HAVE_SQLITE_FTS
+
 static TrackerDBInterface *
 db_interface_get_file_fulltext (void)
 {
@@ -1628,6 +1831,8 @@ db_interface_get_file_fulltext (void)
 	return iface;
 }
 
+#endif /* HAVE_SQLITE_FTS */
+
 static TrackerDBInterface *
 db_interface_get_file_contents (void)
 {
@@ -1654,8 +1859,6 @@ db_interface_get_file_contents (void)
 	return iface;
 }
 
-
-
 static TrackerDBInterface *
 db_interface_get_email_metadata (void)
 {
@@ -1675,6 +1878,8 @@ db_interface_get_email_metadata (void)
 	return iface;
 }
 
+#ifdef HAVE_SQLITE_FTS
+
 static TrackerDBInterface *
 db_interface_get_email_fulltext (void)
 {
@@ -1692,6 +1897,8 @@ db_interface_get_email_fulltext (void)
 	return iface;
 }
 
+#endif /* HAVE_SQLITE_FTS */
+
 static TrackerDBInterface *
 db_interface_get_email_contents (void)
 {
@@ -1735,17 +1942,21 @@ db_interface_create (TrackerDB db)
 	case TRACKER_DB_FILE_METADATA:
 		return db_interface_get_file_metadata ();
 		
+#ifdef HAVE_SQLITE_FTS
 	case TRACKER_DB_FILE_FULLTEXT:
 		return db_interface_get_file_fulltext ();	
+#endif /* HAVE_SQLITE_FTS */
 
 	case TRACKER_DB_FILE_CONTENTS:
 		return db_interface_get_file_contents ();
 
 	case TRACKER_DB_EMAIL_METADATA:
 		return db_interface_get_email_metadata ();
-		
+
+#ifdef HAVE_SQLITE_FTS		
 	case TRACKER_DB_EMAIL_FULLTEXT:
 		return db_interface_get_email_fulltext ();	
+#endif /* HAVE_SQLITE_FTS */
 		
 	case TRACKER_DB_EMAIL_CONTENTS:
 		return db_interface_get_email_contents ();
@@ -1765,8 +1976,11 @@ db_manager_remove_all (void)
 
 	g_message ("Removing all database files");
 
+	/* NOTE: We don't have to be initialized for this so we
+	 * calculate the absolute directories here. 
+	 */
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-		g_message ("Removing database:'%s'",
+		g_message ("  Removing database:'%s'",
 			   dbs[i].abs_filename);
 		g_unlink (dbs[i].abs_filename);
 	}
@@ -1906,12 +2120,19 @@ tracker_db_manager_ensure_locale (void)
 	}
 
 	if (g_strcmp0 (current_locale, stored_locale) != 0) {
+		guint collate_key;
 		/* Locales differ, update collate keys */
-		g_debug ("Updating DB locale dependent data to: %s\n", current_locale);
+		g_message ("Updating DB locale dependent data to: %s\n", current_locale);
 
 		iface = dbs[TRACKER_DB_FILE_METADATA].iface;
 		tracker_db_interface_execute_procedure (iface, NULL, "UpdateMetadataCollation", NULL);
 
+		for (collate_key = 1; collate_key<6; collate_key++) {
+			tracker_db_interface_execute_query (iface, NULL,
+			   		    "UPDATE Services SET KeyMetadataCollation%d=CollateKey(KeyMetadata%d)",
+					    collate_key, collate_key);
+		} 
+
 		iface = dbs[TRACKER_DB_EMAIL_METADATA].iface;
 		tracker_db_interface_execute_procedure (iface, NULL, "UpdateMetadataCollation", NULL);
 
@@ -1921,10 +2142,11 @@ tracker_db_manager_ensure_locale (void)
 	g_free (stored_locale);
 }
 
-void
+gboolean
 tracker_db_manager_init (TrackerDBManagerFlags	flags,
 			 gboolean	       *first_time,
-			 gboolean	        shared_cache)
+			 gboolean	        shared_cache,
+			 const gchar           *profile_name)
 {
 	GType		    etype;
 	TrackerDBVersion    version;
@@ -1938,7 +2160,7 @@ tracker_db_manager_init (TrackerDBManagerFlags	flags,
 	}
 
 	if (initialized) {
-		return;
+		return TRUE;
 	}
 
 	need_reindex = FALSE;
@@ -1975,6 +2197,10 @@ tracker_db_manager_init (TrackerDBManagerFlags	flags,
 				     "tracker",
 				     NULL);
 
+	config_dir = g_build_filename (g_get_user_config_dir (),
+				       "tracker",
+				       NULL);
+
 	filename = g_strdup_printf ("tracker-%s", g_get_user_name ());
 	sys_tmp_dir = g_build_filename (g_get_tmp_dir (), filename, NULL);
 	g_free (filename);
@@ -1982,6 +2208,7 @@ tracker_db_manager_init (TrackerDBManagerFlags	flags,
 	/* Make sure the directories exist */
 	g_message ("Checking database directories exist");
 
+	g_mkdir_with_parents (config_dir, 00755);
 	g_mkdir_with_parents (data_dir, 00755);
 	g_mkdir_with_parents (user_data_dir, 00755);
 	g_mkdir_with_parents (sys_tmp_dir, 00755);
@@ -2032,29 +2259,40 @@ tracker_db_manager_init (TrackerDBManagerFlags	flags,
 		}
 	}
 
+	/* If we are just initializing to remove the databases,
+	 * return here. 
+	 */
+	if ((flags & TRACKER_DB_MANAGER_REMOVE_ALL) != 0) {
+		initialized = TRUE;
+		return TRUE;
+	}
+
 	/* Set general database options */
 	if (shared_cache) {
 		g_message ("Enabling database shared cache");
 		tracker_db_interface_sqlite_enable_shared_cache ();
 	}
 
-	/* Add prepared queries */
-	prepared_queries = g_hash_table_new_full (g_str_hash,
-						  g_str_equal,
-						  g_free,
-						  g_free);
-
+	/* Get prepared queries */
 	load_prepared_queries ();
 
+	/* Get pragma details */
+	load_pragma_file (profile_name);
+
 	/* Should we reindex? If so, just remove all databases files,
 	 * NOT the paths, note, that these paths are also used for
 	 * other things like the nfs lock file.
 	 */
-	if (flags & TRACKER_DB_MANAGER_FORCE_REINDEX || need_reindex) {
+	if ((flags & TRACKER_DB_MANAGER_FORCE_NO_REINDEX) == 0 &&
+	    ((flags & TRACKER_DB_MANAGER_FORCE_REINDEX) != 0 || need_reindex)) {
 		if (first_time) {
 			*first_time = TRUE;
 		}
 
+		if (!tracker_file_system_has_enough_space (data_dir, TRACKER_DB_MIN_REQUIRED_SPACE)) {
+			return FALSE;
+		}
+
 		/* We call an internal version of this function here
 		 * because at the time 'initialized' = FALSE and that
 		 * will cause errors and do nothing.
@@ -2067,8 +2305,6 @@ tracker_db_manager_init (TrackerDBManagerFlags	flags,
 		 * new databases.
 		 */
 		tracker_ontology_shutdown ();
-
-		/* Make sure we initialize all other modules we depend on */
 		tracker_ontology_init ();
 
 		/* Now create the databases and close them */
@@ -2083,10 +2319,23 @@ tracker_db_manager_init (TrackerDBManagerFlags	flags,
 		 * attached to be created correctly.
 		 */
 		for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
-			g_object_unref (dbs[i].iface);
-			dbs[i].iface = NULL;
+			if (dbs[i].iface) {
+				g_object_unref (dbs[i].iface);
+				dbs[i].iface = NULL;
+			}
 		}
+
+		/* Reset ontology - we do this so we repopulate our
+		 * ontology caches with the db_interface_create()
+		 * calls below.
+		 */
+		tracker_ontology_shutdown ();
+		tracker_ontology_init ();
 	} else {
+		if ((flags & TRACKER_DB_MANAGER_FORCE_NO_REINDEX) && need_reindex) {
+			g_message ("Reindex was needed, but has been forbidden by NO_REINDEX flag");
+		}
+
 		/* Make sure we remove and recreate the cache directory in tmp
 		 * each time we start up, this is meant to be a per-run
 		 * thing.
@@ -2097,7 +2346,11 @@ tracker_db_manager_init (TrackerDBManagerFlags	flags,
 			g_unlink (dbs[TRACKER_DB_CACHE].abs_filename);
 		}
 
-		/* Make sure we initialize all other modules we depend on */
+		/* In cases where we re-init this module, make sure
+		 * we have cleaned up the ontology before we load all
+		 * new databases.
+		 */
+		tracker_ontology_shutdown ();
 		tracker_ontology_init ();
 	}
 
@@ -2112,6 +2365,25 @@ tracker_db_manager_init (TrackerDBManagerFlags	flags,
 	tracker_db_manager_ensure_locale ();
 
 	initialized = TRUE;
+
+	return TRUE;
+}
+
+static void
+invalidate_ifaces (void)
+{
+	GList *l;
+	gint i;
+
+	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		if (dbs[i].iface) {
+			g_signal_emit_by_name (dbs[i].iface, "invalidated");
+		}
+	}
+
+	for (l = not_owned_ifaces; l; l = l->next) {
+		g_signal_emit_by_name (l->data, "invalidated");
+	}
 }
 
 void
@@ -2123,6 +2395,8 @@ tracker_db_manager_shutdown (void)
 		return;
 	}
 
+	invalidate_ifaces ();
+
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
 		if (dbs[i].abs_filename) {
 			g_free (dbs[i].abs_filename);
@@ -2135,9 +2409,17 @@ tracker_db_manager_shutdown (void)
 		}
 	}
 
-	g_hash_table_unref (prepared_queries);
-	prepared_queries = NULL;
+	if (prepared_queries) {
+		g_hash_table_unref (prepared_queries);
+		prepared_queries = NULL;
+	}
 
+	if (pragmas) {
+		g_hash_table_unref (pragmas);
+		pragmas = NULL;
+	}
+
+	g_free (config_dir);
 	g_free (data_dir);
 	g_free (user_data_dir);
 	g_free (sys_tmp_dir);
@@ -2171,13 +2453,16 @@ tracker_db_manager_shutdown (void)
 	tracker_ontology_shutdown ();
 
 	initialized = FALSE;
+
+	g_list_free (not_owned_ifaces);
+	not_owned_ifaces = NULL;
 }
 
 void
 tracker_db_manager_remove_all (void)
 {
 	g_return_if_fail (initialized != FALSE);
-
+	
 	db_manager_remove_all ();
 }
 
@@ -2195,6 +2480,10 @@ tracker_db_manager_optimize (void)
 
 	/* Check if any connections are open? */
 	for (i = 1; i < G_N_ELEMENTS (dbs); i++) {
+		if (!dbs[i].iface) {
+			continue;
+		}
+
 		if (G_OBJECT (dbs[i].iface)->ref_count > 1) {
 			g_message ("  DB:'%s' is still open with %d references!",
 				   dbs[i].name,
@@ -2222,6 +2511,13 @@ tracker_db_manager_get_file (TrackerDB db)
 	return dbs[db].abs_filename;
 }
 
+static void
+remove_not_owned_iface (gpointer  user_data,
+			GObject  *iface)
+{
+	not_owned_ifaces = g_list_remove (not_owned_ifaces, iface);
+}
+
 /**
  * tracker_db_manager_get_db_interfaces:
  * @num: amount of TrackerDB files wanted
@@ -2267,6 +2563,11 @@ tracker_db_manager_get_db_interfaces (gint num, ...)
 	}
 	va_end (args);
 
+	if (connection) {
+		not_owned_ifaces = g_list_prepend (not_owned_ifaces, connection);
+		g_object_weak_ref (G_OBJECT (connection), remove_not_owned_iface, NULL);
+	}
+
 	return connection;
 }
 
@@ -2301,6 +2602,11 @@ tracker_db_manager_get_db_interfaces_ro (gint num, ...)
 	}
 	va_end (args);
 
+	if (connection) {
+		not_owned_ifaces = g_list_prepend (not_owned_ifaces, connection);
+		g_object_weak_ref (G_OBJECT (connection), remove_not_owned_iface, NULL);
+	}
+
 	return connection;
 }
 
