@@ -35,6 +35,8 @@
 #include <exempi/xmp.h>
 #include <exempi/xmpconsts.h>
 
+#ifdef ENABLE_DETAILED_METADATA
+
 static gchar *
 fix_metering_mode (const gchar *mode)
 {
@@ -61,6 +63,8 @@ fix_metering_mode (const gchar *mode)
 	return "unknown";
 }
 
+#endif /* ENABLE_DETAILED_METADATA */
+
 static gchar *
 fix_flash (const gchar *flash)
 {
@@ -76,6 +80,8 @@ fix_flash (const gchar *flash)
 	}
 }
 
+#ifdef ENABLE_DETAILED_METADATA
+
 static gchar *
 fix_white_balance (const gchar *wb)
 {
@@ -86,6 +92,8 @@ fix_white_balance (const gchar *wb)
 	}
 }
 
+#endif /* ENABLE_DETAILED_METADATA */
+
 static void tracker_xmp_iter        (XmpPtr          xmp,
 				     XmpIteratorPtr  iter,
 				     GHashTable     *metadata,
@@ -324,6 +332,49 @@ tracker_xmp_iter_simple (GHashTable  *metadata,
 			tracker_append_string_to_hash_table (metadata, "File:License", value, append, FALSE);
 		}
 	}
+	/* XAP (XMP)scheme */
+	else if (strcmp (schema, NS_XAP) == 0) {
+	        if (strcmp (name, "Rating") == 0) {
+		        tracker_append_string_to_hash_table (metadata, "Image:Rating", value, append, FALSE);
+		}
+		if (strcmp (name, "MetadataDate") == 0) {
+		        tracker_append_string_to_hash_table (metadata, "Image:Date", value, append, TRUE);
+		}
+		if (strcmp (name, "ModifyDate") == 0) {
+			tracker_append_string_to_hash_table (metadata, "Image:Date", value, append, TRUE);
+		}
+	}
+	/* IPTC4XMP scheme */
+	else if (strcmp (schema,  NS_IPTC4XMP) == 0) {
+	        if (strcmp (name, "Location") == 0) {
+		        tracker_append_string_to_hash_table (metadata, "Image:Location", value, append, FALSE);
+
+			/* Added to the valid keywords */
+		        tracker_append_string_to_hash_table (metadata, "Image:Keywords", value, TRUE, FALSE);
+
+		}
+		if (strcmp (name, "Sublocation") == 0) {
+		        tracker_append_string_to_hash_table (metadata, "Image:Sublocation", value, append, FALSE);
+
+			/* Added to the valid keywords */
+		        tracker_append_string_to_hash_table (metadata, "Image:Keywords", value, TRUE, FALSE);
+		}
+	}
+	/* Photoshop scheme */
+	else if (strcmp (schema,  NS_PHOTOSHOP) == 0) {
+	        if (strcmp (name, "City") == 0) {
+		        tracker_append_string_to_hash_table (metadata, "Image:City", value, append, FALSE);
+
+			/* Added to the valid keywords */
+		        tracker_append_string_to_hash_table (metadata, "Image:Keywords", value, TRUE, FALSE);
+		}
+		else if (strcmp (name, "Country") == 0) {
+			tracker_append_string_to_hash_table (metadata, "Image:Country", value, append, FALSE);
+
+			/* Added to the valid keywords */
+		        tracker_append_string_to_hash_table (metadata, "Image:Keywords", value, TRUE, FALSE);
+		}
+	}
 	/* Exif basic scheme */
 	else if (strcmp (schema, NS_EXIF) == 0) {
 		if (strcmp (name, "Title") == 0) {
@@ -391,47 +442,6 @@ tracker_xmp_iter_simple (GHashTable  *metadata,
 			tracker_append_string_to_hash_table (metadata, "File:Copyright", value, append, FALSE);
 		}
 	}
-	/* XAP (XMP)scheme */
-	else if (strcmp (schema, NS_XAP) == 0) {
-	        if (strcmp (name, "Rating") == 0) {
-		        tracker_append_string_to_hash_table (metadata, "Image:Rating", value, append, FALSE);
-		}
-		if (strcmp (name, "MetadataDate") == 0) {
-		        tracker_append_string_to_hash_table (metadata, "Image:Date", value, append, FALSE);
-		}
-	}
-	/* IPTC4XMP scheme */
-	else if (strcmp (schema,  NS_IPTC4XMP) == 0) {
-	        if (strcmp (name, "Location") == 0) {
-		        tracker_append_string_to_hash_table (metadata, "Image:Location", value, append, FALSE);
-
-			/* Added to the valid keywords */
-		        tracker_append_string_to_hash_table (metadata, "Image:Keywords", value, TRUE, FALSE);
-
-		}
-		if (strcmp (name, "Sublocation") == 0) {
-		        tracker_append_string_to_hash_table (metadata, "Image:Sublocation", value, append, FALSE);
-
-			/* Added to the valid keywords */
-		        tracker_append_string_to_hash_table (metadata, "Image:Keywords", value, TRUE, FALSE);
-		}
-	}
-	/* Photoshop scheme */
-	else if (strcmp (schema,  NS_PHOTOSHOP) == 0) {
-	        if (strcmp (name, "City") == 0) {
-		        tracker_append_string_to_hash_table (metadata, "Image:City", value, append, FALSE);
-
-			/* Added to the valid keywords */
-		        tracker_append_string_to_hash_table (metadata, "Image:Keywords", value, TRUE, FALSE);
-		}
-		else if (strcmp (name, "Country") == 0) {
-			tracker_append_string_to_hash_table (metadata, "Image:Country", value, append, FALSE);
-
-			/* Added to the valid keywords */
-		        tracker_append_string_to_hash_table (metadata, "Image:Keywords", value, TRUE, FALSE);
-		}
-	}
-
 	g_free (name);
 }
 
